/*
 *  COPYRIGHT (c) 2009 Chris Johns <chrisj@rtems.org>
 *
 *  COPYRIGHT (c) 1989-2009 On-Line Applications Research Corporation (OAR).
 *
 *  The license and distribution terms for this file may be
 *  found in the file LICENSE in this distribution or at
 *  http://www.rtems.com/license/LICENSE.
 *
 *  $Id$
 */
/**
 * @file
 *
 * @ingroup rtems-ext2fs
 *
 * EXT2FS Private Header file.
 *
 * This file is not to be installed. Any EXT2FS type of header file or
 * structure referenced is not to be exported outside of this header or
 * this directory.
 */

#if !defined(RTEMS_EXT2FS_PRIVATE_DEFINED)
#define RTEMS_EXT2FS_PRIVATE_DEFINED

#define RTEMS_EXT2FS_DEBUG_ALL           1
#define RTEMS_EXT2FS_DEBUG_ERROR_MSGS    RTEMS_EXT2FS_DEBUG_ALL
#define RTEMS_EXT2FS_DEBUG_EVAL_PATH     RTEMS_EXT2FS_DEBUG_ALL
#define RTEMS_EXT2FS_DEBUG_EVAL_FOR_MAKE RTEMS_EXT2FS_DEBUG_ALL
#define RTEMS_EXT2FS_DEBUG_EVAL_PERMS    RTEMS_EXT2FS_DEBUG_ALL
#define RTEMS_EXT2FS_DEBUG_MKNOD         RTEMS_EXT2FS_DEBUG_ALL
#define RTEMS_EXT2FS_DEBUG_UNLINK        RTEMS_EXT2FS_DEBUG_ALL
#define RTEMS_EXT2FS_DEBUG_DIR_RMNOD     RTEMS_EXT2FS_DEBUG_ALL
#define RTEMS_EXT2FS_DEBUG_FILE_OPEN     RTEMS_EXT2FS_DEBUG_ALL
#define RTEMS_EXT2FS_DEBUG_FILE_CLOSE    RTEMS_EXT2FS_DEBUG_ALL
#define RTEMS_EXT2FS_DEBUG_FILE_READ     RTEMS_EXT2FS_DEBUG_ALL
#define RTEMS_EXT2FS_DEBUG_FILE_WRITE    RTEMS_EXT2FS_DEBUG_ALL
#define RTEMS_EXT2FS_DEBUG_FILE_LSEEK    RTEMS_EXT2FS_DEBUG_ALL
#define RTEMS_EXT2FS_DEBUG_FILE_FTRUNC   RTEMS_EXT2FS_DEBUG_ALL

#include <ext2fs/ext2fs.h>
#include <ext2fs/ext2_fs.h>
#include <rtems/libio_.h>
#include <rtems/fs.h>

#define RTEMS_EXT2FS_MAX_NAME                    (1 << 8) /* with nul end byte */
#define RTEMS_EXT2FS_NAME_MASK                   (RTEMS_EXT2FS_MAX_NAME - 1)

#define rtems_ext2fs_dev(_loc)                   ((_loc)->mt_entry->fs_info)
#define rtems_ext2fs_handlers(_h)                &rtems_ext2fs_ ## _h ## _handlers
#define rtems_ext2fs_set_pathloc_ino(_loc, _ino) (_loc)->node_access = (void*)(_ino)
#define rtems_ext2fs_get_pathloc_ino(_loc)       ((intptr_t)(_loc)->node_access)
#define rtems_ext2fs_get_iop_ino(_iop)           ((intptr_t)(_iop)->file_info)
#define rtems_extfs_current_dir(_c)              ((_c[0] == '.') && \
                                                  ((_c[1] == '\0') || \
                                                   rtems_filesystem_is_separator(_c[1])))
#define rtems_extfs_parent_dir(_p)               ((_p[0] == '.') && (_p[1] == '.') && \
                                                  ((_p[2] == '\0') || \
                                                   rtems_filesystem_is_separator(_p[2])))

int rtems_ext2fs_imodeflags(mode_t mode);
mode_t rtems_ext2fs_modeflags(int imode);
int rtems_ext2fs_read_inode(ext2_filsys        fs,
                            const char*        path,
                            ext2_ino_t         cwd,
                            ext2_ino_t*        ino,
                            struct ext2_inode* inode);
int rtems_ext2fs_read_parent_inode(ext2_filsys        fs,
                                   ext2_ino_t         ino,
                                   ext2_ino_t*        pino,
                                   struct ext2_inode* pinode);
int rtems_ext2fs_inode_device(struct ext2_inode* inode,
                              int*               major,
                              int*               minor);
int rtems_ext2fs_unlinkbyinode(ext2_filsys        fs,
                               ext2_ino_t         ino, 
                               struct ext2_inode* inode);
int rtems_ext2fs_evaluate_permission(struct ext2_inode* inode, int flags);
int rtems_ext2fs_set_handlers(rtems_filesystem_location_info_t* loc,
                              struct ext2_inode*                inode);
int rtems_ext2fs_remap_error(const char* what, int rc);

/*
 * Shared handlers.
 */
int rtems_ext2fs_stat(rtems_filesystem_location_info_t *loc, struct stat *buf);
int rtems_ext2fs_fchmod(rtems_filesystem_location_info_t *loc, mode_t mode);
int rtems_ext2fs_rmnod(rtems_filesystem_location_info_t *parent_pathloc,
                       rtems_filesystem_location_info_t *pathloc);
int rtems_ext2fs_fdatasync(rtems_libio_t *iop);
int rtems_ext2fs_fcntl(int cmd, rtems_libio_t *iop);

/*
 * I/O Block Manager.
 */
extern io_manager rtems_ext2fs_blockio_manager;
dev_t rtems_ext2fs_blockio_dev(ext2_filsys fs);

/*
 * Handlers for the various supported EXT2FS node operations.
 */
extern const rtems_filesystem_file_handlers_r rtems_ext2fs_dir_handlers;
extern const rtems_filesystem_file_handlers_r rtems_ext2fs_file_handlers;
extern const rtems_filesystem_file_handlers_r rtems_ext2fs_link_handlers;
extern const rtems_filesystem_file_handlers_r rtems_ext2fs_device_handlers;

#endif
